// Comb the whole board, row by row and column by column, checking which values
// are valid in each cell.  If there's only one valid value, then that must
// be the value there.

#include "strategy1.h"

#include <iostream>

bool SudokuStrategy1::operator()(SudokuBoard& board)
{
	bool changed = false;
	possible_list possibilities;

	for(int row = 0; row < SQUARE_INDEX_MAX; row++)
	{
		for(int col = 0; col < SQUARE_INDEX_MAX; col++)
		{
			// Don't bother if there's already a value there
			if(board.get_value(col, row) != -1)
				continue;

			possibilities.clear();
			get_possibilities(board, col, row, possibilities);
			if(possibilities.size() == 1)
			{
				int new_value = *(possibilities.begin());
				std::cout << "Found value " << new_value << " at "
				          << "column " << col << ", row " << row
				          << std::endl;
				board.set_value(col, row, new_value);
				changed = true;
			}
		}
	}
	return changed;
}

void SudokuStrategy1::get_possibilities(const SudokuBoard& board, int col, int row, possible_list& list)
{
	for(int value = 1; value <= SQUARE_VALUE_MAX; value++)
	{
		if(board.is_valid(col, row, value))
			list.push_back(value);
	}
}
