// On a square-by-square basis, loop through each possible value and see
// if there's only one possible spot it could be placed, in which case it
// *must* go in that spot

#include "strategy2.h"

#include <iostream>

bool SudokuStrategy2::operator()(SudokuBoard& board)
{
	bool changed = false;
	possible_list possibilities;

	for(int sqrow = 0; sqrow < SQUARE_SIZE; sqrow++)
	{
		for(int sqcol = 0; sqcol < SQUARE_SIZE; sqcol++)
		{
			bool new_changed = evaluate_square(board, sqcol, sqrow);
			if(new_changed == true)
				changed = true;
		}
	}
	return changed;
}

bool SudokuStrategy2::evaluate_square(SudokuBoard& board, int sqcol, int sqrow)
{
	bool changed = false;

	for(int value = 1; value <= SQUARE_VALUE_MAX; value++)
	{
		possible_list possibilities;
		get_value_locations(board, sqcol, sqrow, value, possibilities);
		if(possibilities.size() == 1)
		{
			coords new_spot = *(possibilities.begin());
			std::cout << "Found a spot for value " << value
				  << " at " << new_spot.first << ","
				  << new_spot.second << std::endl;
			board.set_value(new_spot.first, new_spot.second, value);
			changed = true;
		}
	}
	return changed;
}

// Get all locations within the square where the value could be located

void SudokuStrategy2::get_value_locations(const SudokuBoard& board, int sqcol, int sqrow, int value, possible_list& list)
{
	int start_col = sqcol * SQUARE_SIZE;
	int start_row = sqrow * SQUARE_SIZE;

	for(int col = start_col; col < start_col + SQUARE_SIZE; col++)
	{
		for(int row = start_row; row < start_row + SQUARE_SIZE; row++)
		{
			// Don't bother if there's already a value there
			if(board.get_value(col, row) != -1)
				continue;
			if(board.is_valid(col, row, value))
			{
				coords new_spot(col, row);
				list.push_back(new_spot);
			}
		}
	}
}
