// A brute-force strategy: Comb the whole board, row by row and column by
// column, trying all possible combinations of values.  For each possible
// value, set it and try to solve the rest of the board.  Recursion is used
// to prune paths that lead to invalid board states.

// This method is guaranteed to work, but can take quite a while
// to produce a solution.

#include "strategy3.h"

#include <iostream>

bool SudokuStrategy3::operator()(SudokuBoard& board)
{
	try
	{
		board = solve(board);
	}
	catch(...)
	{
		return false;
	}

	return true;
}

// The basic strategy is to find an empty cell, fill it with a
// potential value, and then solve the rest of the board recursively.
// Returns a valid solution if one is found, or throws an
// exception if a dead end (no more valid values can be placed)
// or invalid state is reached.

SudokuBoard SudokuStrategy3::solve(SudokuBoard& board)
{
	// Get the next empty cell
	int col, row;
	try
	{
		board.next_empty(col, row);
	}
	catch(int)
	{
		std::cout << "Board is full without solution" << std::endl;
		throw -1;
	}

	// Try all possible values in that cell
	for(int i = 1; i <= SQUARE_VALUE_MAX; i++)
	{
		SudokuBoard board_copy = board;
		if(board_copy.is_valid(col, row, i))
		{
			std::cout << "Trying " << i << " at " << col << ","
				<< row << std::endl;
			board_copy.set_value(col, row, i);
			if(board_copy.is_solved())
			{
				// Yay, a solution!
				return board_copy;
			}

			// Now try solving the rest of the board
			try
			{
				board_copy = solve(board_copy);
				// If we got a board back, it's a solution, since
				// it would have thrown an exception otherwise
				return board_copy;
			}
			catch(...)
			{
				// An exception indicates a solution couldn't be
				// found, but let it continue trying the rest
				// of the possible values
			}
		}
	}

	// Exhausted all possible values for that cell without a solution,
	// so we've hit a dead end.
	std::cout << "Hit a dead end, backtracking" << std::endl;
	throw -1;
}
